/*
** Injection Framework - Automatized Sql Injection Tool
** Copyright (C) 2006 Flowgate Security Consulting,
**         http://www.flowgate.net
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License as published by the
** Free Software Foundation; either version 2 of the License, or (at your
** option) any later version.
** 
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** The GNU General Public License is available from http://www.gnu.org/ or
** the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA.
**
** For information on Injection Framework: fg_injector@flowgate.net

*/
#ifndef _INFERENCE_H_
#define _INFERENCE_H_
//#if defined(__GNUG__) && !defined(__APPLE__)
//#pragma interface "inference.cpp"
//#endif
#include "wx/socket.h"
#include "wx/datetime.h"
#include "myframe.h"
#include "utils.h"
#include "states.h"
#include "types.h"
#include <list>
#include <vector>
#include "myclient.h"
#include "utils.h"


class BitMaskString:public Object{
public:
	//friend wxString& operator<<(wxString& stream, BitMaskString& string);
	wxString mString;
	wxString mMask;
	wxString mReserved;
	wxString GetString();
	void Clear();
	void ReserveBit(int absoluteBitPos);
	void SetBit(int absoluteBitPos, bool value);
	bool IsNullterminated();
	int FirstNotReserved();
};
class InferenceEngine:public Object{
private:
	wxTextCtrl* mpOutput;
	wxString mRequest;
	wxString mTrueStr;
	EngineView *mpView;
	wxString mFalseStr;
	wxString mTrueKey;
	wxString mFalseKey;
	wxString mIndex;
	wxString mPort, mHostname;
	vector<wxString> mQueryFields;
	wxString mPostSql;
	bool mRunning;
	bool mAddQuotes;
	bool mHasWhere;
	bool mIsInitializing;
	wxString mTrueRequest;
	wxString mFalseRequest;
	wxString mDiferenciationString;
	wxString mFromSql;
	wxString mWhereCond;
	int mNumberForTrue;
	int mNumberForFalse;
	int mSequenceMin;
	int mEventThreads;
	int mSequence; //the number of times StartInjection was called
	int mNumConnections;
	int mFieldNumber; //when doing a set inference, the current field
	wxString mBaseIndex; //when doing a set inference, the index currently infering
	unsigned int mLastResultLength;
	wxString mQuery, mKeyName;
	BitMaskString mResult; 

    static bool mInstanceFlag;
    static InferenceEngine *mSingle;
	InferenceEngine();
	bool DecideDiferenciation();
	int GetValue(wxString html, bool &value);
	wxString GetRequest(int bitPosition);
	void MakeInjection();
	void Continue(wxString const request, int sequence, wxString index, int fieldNumber, int bitPosition);
	wxString SqlCharacters(wxString string);
public:
	enum InferenceType {ENG_AUTO, ENG_BYKEY};
	enum InjectionType {ENG_VALUES, ENG_CONJ};
	enum DatabaseType {ENG_MSSQL, ENG_ORACLE, ENG_MYSQL, ENG_PSSQL};
	enum ResultType {ENG_SINGLE, ENG_LIST};
	InjectionType mInjectionType;
	DatabaseType mDatabaseType;
	InferenceType mInferenceType;
	ResultType mResultType;
	wxString Initialize();

	static InferenceEngine* GetInstance();
	wxString GetStateMsg();
	void Stop();
	void SetOptions(InferenceType inference, 
					ResultType resultType,
					InjectionType injectionType,	
					wxString trueStr, 
					wxString falseStr, 
					wxString trueKey, 
					wxString falseKey, 
					wxString query, 
					wxString keyName,
					wxTextCtrl *outputStream, 
					wxString hostname, 
					wxString port, 
					wxString request, 
					DatabaseType databaseType, 
					int numConnections,
					EngineView *view,
					bool addQuotes);
	bool IsRunning(){return mRunning;}
	//returns false if it's already running or
	//it doesn't have enough data.
	wxString StartInjections();
	static void SocketCallback(MyClient* client, void* clientData );

};
struct ClientInfo:public Object{
	int mBitPosition;
	int mSequence;
	int mFieldNumber;
	wxString mBaseIndex;
};
wxString escape(wxString str);
#endif
	//endif _INFERENCE_H_
