/*
** Injection Framework - Automatized Sql Injection Tool
** Copyright (C) 2006 Flowgate Security Consulting,
**         http://www.flowgate.net
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License as published by the
** Free Software Foundation; either version 2 of the License, or (at your
** option) any later version.
** 
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** The GNU General Public License is available from http://www.gnu.org/ or
** the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA.
**
** For information on Injection Framework: fg_injector@flowgate.net

*/
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "myclient.h"
#include "dnscache.h"
#include "options.h"
//#include "pro1.h"

#include <vector> 
#include <string> 

//--------------------------------------------------------------------------- 
/// Converts from wxString (ANSI or Unicode) to UTF8 
std::string ToUTF8(const wxString &s) 
{ 
    return std::string((const char *) s.mb_str(wxConvUTF8)); 
} 
//--------------------------------------------------------------------------- 
/// Converts from ANSI (wxConvLibc) to UTF8 
std::string ToUTF8(const std::string &s) 
{ 
    // Convert to array of wchar_t's to minimize conversion errors 
    size_t szOutWC = wxConvLibc.MB2WC(NULL, s.c_str(), 0); 
    std::vector<wchar_t> wchBuffer(szOutWC+1); 
    wxConvLibc.MB2WC(&*wchBuffer.begin(), s.c_str(), (size_t) wchBuffer.size()); 
    // Convert from wchar_t's to array of chars to minimize conversion errors 
    size_t szOutMB = wxConvUTF8.WC2MB(NULL, &*wchBuffer.begin(), 0); 
    std::string sOut(szOutMB, '?'); 
    // The size + 1 is perfectly legal here because the string always includes a terminating null character 
    wxConvUTF8.WC2MB(&*sOut.begin(), &*wchBuffer.begin(), szOutMB+1); 
    return sOut; 
} 

/// Converts from UTF8 to wxString (ANSI or Unicode) 
wxString FromUTF8(const std::string &s) 
{ 
    // When conversion to wxString failed because the string is not 
    // of type UTF8 (coding error), it falls back to conversion from 
    // ANSI to wxString 

    if (wxConvUTF8.MB2WC(NULL, s.c_str(), s.size())==(size_t)-1) { 
        return wxString(ToUTF8(s).c_str(), wxConvUTF8); 
    } 
    return wxString(s.c_str(), wxConvUTF8); 
} 



extern MyFrame *mainWindow;
bool MyClientCollection::mInstanceFlag = false;
MyClientCollection* MyClientCollection::mSingle = NULL;
MyClientCollection* MyClientCollection::GetInstance()
{
	if(! mInstanceFlag)
	{
		mSingle = new MyClientCollection();
		mInstanceFlag = true;
		return mSingle;
	}
	else return mSingle;
}
MyClient::~MyClient(){
}
MyClientCollection::MyClientCollection(){
	wxTheApp->Connect(ID_SOCKET2, wxEVT_SOCKET, (wxObjectEventFunction) &MyClientCollection::OnSocketEvent, NULL, this);
}
MyClient* MyClientCollection::GetClient(){
	MyClient* client = new MyClient;
	wxSocketClient *pSocket = new wxSocketClient();
	client->mpSocket = pSocket;
	pSocket->SetEventHandler(*((wxEvtHandler*)this), ID_SOCKET2); 
	return client;
}
void MyClientCollection::OnSocketEvent(class wxSocketEvent& evt){
	wxSocketBase* cSocket = evt.GetSocket(); 
	MyClient *client = (MyClient*) cSocket->GetClientData();
	int event = evt.GetSocketEvent();
	switch (event){
	case wxSOCKET_INPUT:

		client->OnInput();
		break;
	case wxSOCKET_OUTPUT:
		client->OnOutput();
		break;
	case wxSOCKET_CONNECTION:
		break;
	case wxSOCKET_LOST:
		client->OnClose();
		break;
	}
}
void MyClient::OnInput(){
	char *buf = new char[32000];
	mpSocket->Read(buf, 32000);
	#if wxUSE_UNICODE
	wxString s((const char*)buf, wxConvUTF8, (size_t) mpSocket->LastCount());
	if(!s.length()){
		wxString siso((const char*)buf, wxConvISO8859_1, (size_t) mpSocket->LastCount());
		s = siso;
	}
	#else
	wxString s((const char*)buf, (size_t) mpSocket->LastCount());
	#endif
	if(mAnswer!=wxT(""))
		mAnswer += s;
	else
		mAnswer = s;
	delete buf;
}
void MyClient::OnClose(){
	mpSocket->Destroy();
	wxTextCtrl* logctl = wxDynamicCast(mainWindow->FindWindow(LASTRESPONSETXTCTL), wxTextCtrl);
	logctl->SetValue(mAnswer);
	(*mpCallbackFunction)(this, mpClientData);
}
void MyClient::OnOutput(){
	char *buf = new char[mData.length()];
	for(size_t i=0; i<mData.length(); i++)
		buf[i]=(char) mData[i];
	mpSocket->Write(buf, mData.length());
	mData = mData.Mid(mpSocket->LastCount());
	delete buf;
}
void MyClient::DoRequest(){
	wxTextCtrl* logctl = wxDynamicCast(mainWindow->FindWindow(LASTREQUESTTXTCTL), wxTextCtrl);
	logctl->SetValue(mData);
	mpSocket->SetTimeout(10);
	mpSocket->SetFlags(wxSOCKET_NOWAIT);
	mpSocket->SetClientData(this);
	wxIPV4address *addr;
	if(Options::sUseProxy){
		ProxyData(mHost, mPort);
		addr=GetwxIPV4address(Options::sProxyHost,Options::sProxyPort);
	} else {
		addr=GetwxIPV4address(mHost,mPort);
	}
	
	mpSocket->Connect(*addr, false);
	mpSocket->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG|wxSOCKET_LOST_FLAG|wxSOCKET_OUTPUT_FLAG);
	mpSocket->Notify(true);
}
void MyClient::SetRequestData(wxString host, wxString port, wxString data, void (*callbackFunction)(MyClient* client , void* clientData), void *clientData){
	mHost = host;
	mPort = port;
	mData = data;
	mpCallbackFunction = callbackFunction;
	mpClientData = clientData;
}
//change the first header line to send to a proxy
void MyClient::ProxyData(wxString host, wxString port){
	size_t endline_pos = mData.find_first_of(wxString::FromAscii("\n"));
	wxString first_line = mData.Mid(0,endline_pos);
	size_t pos = first_line.find_first_of(wxString::FromAscii(" \t"));
	if(pos != wxString::npos){
		size_t uri_start = first_line.find_first_not_of(wxString::FromAscii(" \t"), pos);
		if(uri_start != wxString::npos){
			wxString s_prehost = first_line.Mid(0,uri_start);
			wxString s_posthost = first_line.Mid(uri_start);
			first_line = s_prehost + wxString::FromAscii("http://")+host+wxString::FromAscii(":")+port+ s_posthost;
		}
	}
	mData = first_line + mData.Mid(endline_pos);
}
