/*
** Injection Framework - Automatized Sql Injection Tool
** Copyright (C) 2006 Flowgate Security Consulting,
**         http://www.flowgate.net
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License as published by the
** Free Software Foundation; either version 2 of the License, or (at your
** option) any later version.
** 
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** The GNU General Public License is available from http://www.gnu.org/ or
** the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA.
**
** For information on Injection Framework: fg_injector@flowgate.net

*/
#include "wx/wxprec.h"
//#ifdef __BORLANDC__
//#pragma hdrstop
//#endif
#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "myserver.h"
#include "options.h"
#include "dnscache.h"
//#include "pro1.h"

extern MyFrame *mainWindow;

MyServer::MyServer(){
}
MyServer::~MyServer(){
}

void MyServer::SetOptions(bool intercept, bool excludeNonhtml){
	mIntercept = intercept;
	mExcludeNonhtml = excludeNonhtml;
}
void  MyServer::OnSocketEvent(class wxSocketEvent& evt){ 
	//::wxLogDebug(_("OnSocketEvent"));
	wxSocketBase* pSocket = evt.GetSocket(); 
	ProxyDataPort *proxyData = (ProxyDataPort*)pSocket->GetClientData();
	int event = evt.GetSocketEvent();
	switch (event){
	case wxSOCKET_INPUT:
		{
			char *buf = new char[32000];
			pSocket->Read(buf, 32000);
			#if wxUSE_UNICODE
			wxString s((const char*)buf, wxConvUTF8, (size_t) pSocket->LastCount());
			if(!s.length()){
				wxString siso((const char*)buf, wxConvISO8859_1, (size_t) pSocket->LastCount());
				s = siso;
			}
			#else
			wxString s(buf, pSocket->LastCount());
			#endif
			proxyData->OnInput(s);
			delete buf;
		}
		break;
	case wxSOCKET_OUTPUT:
		break;
	case wxSOCKET_CONNECTION:
		break;
	case wxSOCKET_LOST:
		proxyData->OnClose();
		break;
	}
	if(proxyData->mProxy) proxyData->mProxy->Continue(this);
	if (!proxyData->Continue(this)) {
		wxDateTime now = wxDateTime::Now();
		wxTimeSpan span = now - StartingTime;
		ProxysPendigDelete.push_back(pair<wxLongLong, ProxyDataPort*>(span.GetSeconds(), proxyData));
		if(proxyData->mProxy){
			ProxysPendigDelete.push_back(pair<wxLongLong, ProxyDataPort*>(span.GetSeconds(), proxyData->mProxy));
		}
	}

} 
bool MyServer::mInstanceFlag = false;
MyServer* MyServer::mSingle = NULL;
MyServer* MyServer::GetInstance()
{
	if(!mInstanceFlag)
	{
		mSingle = new MyServer();
		mInstanceFlag = true;
		return mSingle;
	}
	else return mSingle;
}
void MyServer::SendNextRequest(){
	if(mProxyList.size()){
		ProxyDataPort* proxy = mProxyList.front();
		mProxyList.pop_front();
		wxTextCtrl* ctl = wxDynamicCast(mainWindow->FindWindow(PROXYTXTCTL), wxTextCtrl);
		if (mProxyList.size()){
			ProxyDataPort* p_first_proxy = mProxyList.front();
			wxASSERT_MSG(p_first_proxy->mRequest, wxT("mRequest is NULL"));
			ctl->SetValue(p_first_proxy->mRequest->mData);
		} else {
			ctl->SetValue(wxString::FromAscii(""));
		}
		wxASSERT_MSG(proxy->mType == PRX_SERV, wxT("prx_serv	expected")); 
		proxy->mState = PRX_STANDBY;
		proxy->Continue(this);
	}
}
void MyServer::Dump(){
	wxString s;
	//fix this on gcc
	//s<<typeid(*this).name();
	s<<wxT("ProxysPendigDelete ")<<ProxysPendigDelete.size();
	s<<wxT(" mProxyList ")<<mProxyList.size();
	//wxLogDebug(s);
}

void MyServer::MySetupSocketEvents(wxSocketServer* pServerSocket, wxEvtHandler* pHandler, void (*onNewRequest)(wxString), void (*onCloseRequest)()) 
{ 
	mOnNewRequest = onNewRequest;
	mOnCloseRequest = onCloseRequest;
	wxASSERT_MSG(pHandler != NULL || wxTheApp != NULL, 
						wxT("NO EVENT HANDLER!")); 

	if (pHandler == NULL)pHandler = wxTheApp; 

	pServerSocket->SetEventHandler(*pHandler, ID_SERVER); 
	pHandler->Connect(ID_SERVER, wxEVT_SOCKET, 
		(wxObjectEventFunction) &MyServer::OnServerEvent, NULL, this); 
	pHandler->Connect(ID_SOCKET, wxEVT_SOCKET, 
		(wxObjectEventFunction) &MyServer::OnSocketEvent,NULL, this); 

	pServerSocket->SetNotify(wxSOCKET_CONNECTION_FLAG); 
	pServerSocket->Notify(true); 
	StartingTime = wxDateTime::Now();
}
ObjHttpRequest* MyServer::GetCurrentRequest(){
	if (mProxyList.size()){
		ProxyDataPort* proxy = mProxyList.front();
		return proxy->mRequest;
	}
	else return NULL;
}
wxSocketServer* MyServer::CreateMyServerSocket(int nPort)
{ 
	wxIPV4address addr; 
	if (!addr.Service(nPort)) return NULL; 
	wxSocketServer* pServerSocket = new wxSocketServer(addr); 
	if (!pServerSocket || !pServerSocket->Ok())
	{
		wxString m=wxT("");
		m<<wxT("Port ")<<nPort<<wxT(" is in use!");
		wxMessageBox(m);
		if(pServerSocket) pServerSocket->Destroy();
		return NULL;
	}
	return pServerSocket;
}
void MyServer::DeleteOldObjects(){
	list<pair<wxLongLong, ProxyDataPort*> >::iterator iterator;
	iterator = ProxysPendigDelete.begin();
	wxDateTime now = wxDateTime::Now();
	wxTimeSpan span = now - StartingTime;
	while(iterator != ProxysPendigDelete.end() && iterator->first<span.GetSeconds() - SOCKET_TIMEOUT){
		
		delete iterator->second;
		ProxysPendigDelete.remove_if(EqualBySecond<wxLongLong, ProxyDataPort*>(*iterator));
		//ProxysPendigDelete.erase(iterator);

		iterator = ProxysPendigDelete.begin();
	}
}
void MyServer::OnServerEvent(class wxSocketEvent& evt) 
{
	//::wxLogDebug(_("OnServerEvent"));
	//contract checking and logging
	wxASSERT(evt.GetSocketEvent() == wxSOCKET_CONNECTION); 
	DeleteOldObjects();
	//implementation
	//create socket and make asociated objects
	wxSocketServer* pServerSocket = (wxSocketServer*) evt.GetSocket(); 
	wxSocketBase *  sSocket = pServerSocket->Accept(false); 
	if (!sSocket) return; 
	sSocket->SetFlags(wxSOCKET_NOWAIT);
	ProxyDataPort *sProxy = new ProxyDataPort;
	sProxy->mRequest = new ObjHttpRequest;
	//todo nonblocking
	sProxy->mState = PRX_START;
	sProxy->mType = PRX_SERV;
	sProxy->mpSocket = sSocket;
	sSocket->SetClientData(sProxy);
	//setup for socket events
	sSocket->SetEventHandler(*((wxEvtHandler*)this), ID_SOCKET); 
	sSocket->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG|wxSOCKET_LOST_FLAG|wxSOCKET_OUTPUT_FLAG); 
	sSocket->Notify(true);
}
void ProxyDataPort::Dump(){
	wxString s;
	//s<<wxT(" mState ")<<GetStringProxyState(mState);
	//s<<wxT(" mType ")<<GetStringProxyType(mType);
	//s<<wxT("mData ")<<mData;
	Object::Dump(s);
}
ProxyDataPort::ProxyDataPort(){
	mType = PRX_UNDEF_TYPE;
	mState = PRX_UNDEF_STATE;
	mData = wxT("");
	mRequest = NULL;
	mProxy = NULL;
}
ProxyDataPort::~ProxyDataPort(){
	if (mRequest) delete mRequest;
}
//returns true if the object is ready to be deleted.
int ProxyDataPort::Continue(MyServer* server){
	//::wxLogDebug(_("Continue"));
	if(mType == PRX_SERV && mState == PRX_START){
		bool header_is_complete = mRequest->IsHeaderComplete();
		mRequest->mProxyHostname = mRequest->GetHostname();

		mRequest->mProxyService = mRequest->GetService();
		if(header_is_complete && mRequest->mProxyHostname.Length() && mRequest->mProxyService.Length()){
			mRequest->ProcessHeaders();
			if(!Options::sUseProxy) mRequest->mData = mRequest->GetUnproxyData();
			mData = mRequest->mData;
			if(server->mIntercept && (!server->mExcludeNonhtml || mRequest->IsHtml())){
				mState = PRX_WAITING;
				if(!server->mProxyList.size()){
					if(!wxTheApp->IsActive()){
						mainWindow->RequestUserAttention();
					}
					wxTextCtrl* ctl = wxDynamicCast(mainWindow->FindWindow(PROXYTXTCTL), wxTextCtrl);
					ctl->SetValue(Options::sUseProxy?mRequest->GetUnproxyData():mRequest->mData);
				}
				server->mProxyList.push_back(this);
			}
			else mState = PRX_STANDBY;
		}
	}
	if(mType == PRX_SERV && mState == PRX_STANDBY){
		if(mRequest->mProxyHostname.Length() && mRequest->mProxyService.Length()){
			mState = PRX_SENDING;
			wxIPV4address *addr;
			if(Options::sUseProxy){
				addr=GetwxIPV4address(Options::sProxyHost,Options::sProxyPort);
			} else {
				addr=GetwxIPV4address(mRequest->mProxyHostname,mRequest->mProxyService);
			}
			DoConnection(*addr, server);
		}
	}
	wxString msg;
	msg.Printf(_("mType: %s datalength %d proxy %d"),GetStringProxyType(mType) , mData.length(), mProxy);
	//::wxLogDebug(msg);
	if ((mState == PRX_SENDING || mState == PRX_CLOSING) && mProxy && mProxy->mpSocket && mData.length()){
		char *buf = new char[mData.length()];
		for(size_t i=0; i<mData.length(); i++)
			buf[i]=(char) mData[i];
		mProxy->mpSocket->Write(buf, mData.length());
		delete buf;
		wxString msg;
		msg.Printf(_("bytes written: %d"), mProxy->mpSocket->LastCount());
		//::wxLogDebug(msg);
		mData = mData.Mid(mProxy->mpSocket->LastCount());
	}


	if (mState == PRX_CLOSING){
		if (!mProxy || mProxy->mState == PRX_CLOSING || mData.length() == 0 ){
			if (mProxy){
				if(mProxy->mpSocket) mProxy->mpSocket->Destroy();
				mProxy->mProxy = NULL;
				mProxy->mpSocket = NULL;
			}
			if(mpSocket){
				mpSocket->Destroy();
				mpSocket = NULL;
			}
			return 0;
		}
	}
	return 1;
}
void ProxyDataPort::OnClose(){
	//::wxLogDebug(_("OnClose"));
	mState = PRX_CLOSING;
}
void ProxyDataPort::DoConnection(wxIPV4address &addr, wxEvtHandler* evtHandler){
	//::wxLogDebug(_("DoConnection"));
	wxSocketClient *cSocket = new wxSocketClient();
	ProxyDataPort *cProxy = new (ProxyDataPort);
	cProxy->mState = PRX_SENDING;
	cProxy->mType = PRX_CLIENT;
	cProxy->mpSocket = cSocket;
	cProxy->mProxy = this;
	cSocket->SetClientData(cProxy);
	cSocket->SetTimeout(10);
	cSocket->SetFlags(wxSOCKET_NOWAIT);
	cSocket->SetEventHandler(*evtHandler, ID_SOCKET); 
	cSocket->Connect(addr, false);
	cSocket->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG | wxSOCKET_LOST_FLAG | wxSOCKET_OUTPUT_FLAG);
	cSocket->Notify(true);
	//asociate mProxy data
	mProxy = cProxy;
}
void ProxyDataPort::OnInput(wxString data){
	//::wxLogDebug(_("OnInput"));
	mData.Append(data);
	if (mType == PRX_SERV){
		mRequest->mData.Append(data);
	}
}
